#include <stdio.h>
#include <stdlib.h>

#define NUM_DIGITS 20

void Initialize(int *array, int num_elements);

/* In printing the array, you DO NOT want to change the array, just output
it. */
void Print_Array(const int array[], int num_elements);
int  Return_Median(int *array, int num_elements);
void Sort_Array(int *array, int num_elements);
void Swap(int *i, int *j);

int main() {
    /* Array that is initialized by a list given to it. */
    int numbers[] = {1, 4, 78, 34, 65, 53, 5, 2, 34, 1, 10};
    int num_numbers = 11; /* I counted the numbers. Not elegant, but easy!*/
    /* Uninitialized array */
    int dummy[NUM_DIGITS];
    int digits[NUM_DIGITS]; 
    int i,j,k;		/* Loop and temp variables */
    int median;

    /* TEST 1 : Passing arrays into functions */
    /* Initialize the digits array */
    Initialize(digits, NUM_DIGITS);
    Initialize(dummy, NUM_DIGITS);
    Print_Array(digits, NUM_DIGITS);
    /* Change the array, arbitrarily */
    for(i=0; i < NUM_DIGITS; i++) { digits[i] = i; }
    /* Print the digits array */
    Print_Array(digits, NUM_DIGITS);
    /* Is there a difference between this and the first call?? */
    Initialize(&digits[0], NUM_DIGITS);
    /* Printing tha array will show us if there is. */
    Print_Array(digits, NUM_DIGITS);

    /* What happens if we use the same type of passing as the second call
	of Initialize? */
    Print_Array(&digits[0], NUM_DIGITS);

    /* Finally, what if we change the array, and pass &digits[1] or a
	different &digits[INT]? */
    for(i=0; i < NUM_DIGITS; i++) { digits[i] = i; }
    printf("Mem. addr. passed\tj:\tPrint_Array(&digits[j],NUM_DIGITS-j) results:\n");
    for(j=1; j < NUM_DIGITS; j++) {
	printf("%p\t%d\t", &digits[j],j);
	Print_Array(&digits[j], NUM_DIGITS-j);
    }
    /* MORAL OF TEST 1 (prove this to yourself!!!) When you are passing an
	array, you are
     * simply passing a memory address.  */

    /* TEST 2: Changing the array for a better purpose. */
    /* We want to find the median element of the numbers array (explained
	in the function
     * definition of Return_Median(). */
    median = Return_Median(numbers, num_numbers); /* Could also pass &numbers[0] */
    printf("The median of numbers[] is %d\n",median);

    return 1;
}


void Initialize(int *array, int num_pos) {
    int i;
    /* This is an initialization loop for letters[]. */
    for(i=0; i < num_pos; i++) {
	array[i] = 0;
    }
}

void Print_Array(const int array[], int num_elements) {
    int i;

    for(i=0; i<num_elements; i++) {
	printf("%2d ",array[i]);
    }
    printf("\n");

}

/* The median of a set of numbers is the middle element of a sorted list
 * of elements.  For example, in the list
 *	1 1 2 5 7 7 9 13 45 67 202
 * the median element is 9.  This function returns the median.
 */
int Return_Median(int *array, int num_elements) {
    /* Notice that even though I passed in the array numbers, it is now defined as int *array.
     * int *array holds the memory address of &numbers[0].  We can make changes to int *array
     * and these changes will be equivalent to changing numbers[] */
    int median;	/* The value we want to return */
    int middle = num_elements / 2; /* This will be a return value of int for all cases, 
				    * which is what we want since it will be a reference 
				    * to an element in array. */

    /* To find the median, we would like to have a sorted array.  So, we will first sort the
     * array of numbers that we are interested in finding the median for. */
    Sort_Array(&array[0], num_elements);
    median = array[middle];  /* In the sorted list, the middle element is the median. */

    return median;
}

/* We will use the bubble_sort to sort the array.
 * TAKE THIS CODE AND IMPLEMENT THE INSERTION SORT!!  */
void Sort_Array(int *array, int num_elements) {
    /* Again, this is receiving the memory address of array from Return_Median, which holds
     * the memory address of numbers from the main program.  Prove this to yourself! */
    int done, i;

    printf("Intermediate Stages of the sorting routine (Bubble Sort):\n");
    done = 0; /* We are not done.  So, in the while condition, !done evaluates to 1, thus
	       * keep going through the while loop.  */
    while(!done) {
	done = 1; /* initialize done to 1.  If no changes are made, done will stay equal to
		   * 1, and the while loop will end.  If changes are made, we change done to
		   * 0, and keep iterating through the while loop. */
        for(i=0; i < num_elements - 1; i++) {
	    if(array[i] > array[i+1]) {
	        Swap(&array[i],&array[i+1]);
	        Print_Array(array,num_elements);
		done = 0;  /* We made a change, so we will have to go through the array of
			    * numbers another time to test if we have truly finished */
	    }
        }
    }

    printf("Sorted array:\n");
    Print_Array(array,num_elements);
}

void Swap(int *element_1, int *element_2) {
    int tmp;
    tmp = *element_1;
    *element_1 = *element_2;
    *element_2 = tmp;
}
