#include <stdio.h>

#define NUM_AA 21
#define NUM_TRAIN 100
#define NUM_TEST 10
#define MAX_SEQ_SIZE 500
#define MAX_ID 15

double train_aa_freq[NUM_AA];
char test_id[NUM_TEST][MAX_ID];
double test_aa_freq[NUM_TEST][NUM_AA];
double sum_of_squares[NUM_TEST];

typedef enum{ala,arg,asn,asp,cys,gln,glu,gly,his,ile,leu,lys,met,phe,pro,
             ser,thr,trp,tyr,val,oth}        
amino_acid_t;

void Initialize_Arrays(void);
void Read_Sequences(double *train_aa_freq);
void Print_Squences(double *train_aa_freq);
void Read_Test_Sequences(char test_id[NUM_TEST][MAX_ID], double test_aa_freq[NUM_TEST][NUM_AA]);
void Print_Test_Freq(void);
void Print_Results(void);
int AA(char aa);


int main(void){
    Initialize_Arrays();
    Read_Sequences(&train_aa_freq[0]);
    Print_Squences(&train_aa_freq[0]);    
    Read_Test_Sequences(test_id/*[NUM_TEST][MAX_ID]*/, test_aa_freq/*[NUM_TEST][NUM_AA]*/);    
    Print_Test_Freq();
    Print_Results();
    return 0;
}



void Initialize_Arrays(void){
    int i,j;

    for(i=0;i<NUM_AA;i++){
        train_aa_freq[i]=0;
    }
    for(i=0;i<NUM_TEST;i++){
        for(j=0;j<MAX_ID;j++){
            test_id[i][j]='\0';
        }
    }
    for(i=0;i<NUM_TEST;i++){
        for(j=0;j<NUM_AA;j++){
            test_aa_freq[i][j]=0;           
	}
    }	        
    for(i=0;i<NUM_TEST;i++){
        sum_of_squares[i]=0;
    }
}

void Read_Sequences(double *train_aa_freq){
     int i=0,j= -1;
     char aa;
     FILE *in;
     in=fopen("train.dat","r");
     while(fscanf(in,"%c",&aa)!=EOF){
        if (aa=='>'){
	    j++;
            while(aa != '\n') {
	  	fscanf(in,"%c",&aa);
            }
        }else{
            int acid=AA(aa);
            //train_aa_freq[j][acid]++;
            train_aa_freq[acid]++;
	    i++;
        }
    }
    for(j=0;j<NUM_AA;j++){
	train_aa_freq[j] /= i;
    }
}

void Read_Test_Sequences(char test_id[NUM_TEST][MAX_ID], double test_aa_freq[NUM_TEST][NUM_AA]) {
    int j,q;
    int test_aa_increment[NUM_TEST];
    char aa;
    FILE *in;
    in=fopen("test.dat","r");
    //initialize test_aa_increment
    for(j=0;j<NUM_TEST;j++){
	test_aa_increment[j]=0;
    }
    j=-1;
    q=0;
    while((fscanf(in,"%c",&aa))!=EOF){
        if (aa=='>'){
	    j++;
	    q=0;
            while(aa != '\n') {
		test_id[j][q]=aa;
	  	fscanf(in,"%c",&aa);
		q++;
            }
        }else{
	    int acid=AA(aa);
            test_aa_freq[j][acid]++;
	    test_aa_increment[j]++;
        }
    }

    for(j=0;j<NUM_TEST;j++){
	for(q=0;q<NUM_AA;q++){
	    test_aa_freq[j][q] /= test_aa_increment[j];
	    sum_of_squares[j]+=(test_aa_freq[j][q]-train_aa_freq[q])*(test_aa_freq[j][q]-train_aa_freq[q]);
	}
    }
}

void Print_Squences(double *train_aa_freq){
    int i;
    for(i=0;i<NUM_AA;i++){
	printf("%2d: %1.2f\n",i+1,train_aa_freq[i]);
    }
}

void Print_Test_Freq(void){
    char amino_acids[120]="A     R     N     D     C     Q     E     G     H     I     L     K     M     F     P     S     T     W     Y     V";
   
    int i,j;
    for(i=0;i<NUM_TEST;i++){
	printf("\n%s\n",test_id[i]);
	printf("     %s\n",amino_acids);
	for(j=0;j<NUM_AA-1;j++){
	    printf("%6.2f",test_aa_freq[i][j]);
	}
    } 
}

void Print_Results(void){
    int i,j,out;
    double lowest;
    printf("\n");
    for(i=0;i<NUM_TEST;i++){
	lowest=99999;
	for(j=0;j<NUM_TEST;j++){
	    if(sum_of_squares[j] < lowest){
		lowest=sum_of_squares[j];
		out=j;
	    }
	}
	printf("%s\n",test_id[out]);
	sum_of_squares[out]=99999;
    }
}

int AA(char aa){
    switch(aa){
    case 'A':
         return ala;
         break;
    case 'R':
         return arg;
         break;
    case 'N':
         return asn;
         break;
    case 'D':
         return asp;
         break;
    case 'C':
         return cys;
         break;
    case 'Q':
         return gln;
         break;
    case 'E':
         return glu;
         break;
    case 'G':
         return gly;
         break;
    case 'H':
         return his;
         break;
    case 'I':
         return ile;
         break;
    case 'L':
         return leu;
         break;
    case 'K':
         return lys;
         break;
    case 'M':
         return met;
         break;
    case 'F':
         return phe;
         break;
    case 'P':
         return pro;
         break;
    case 'S':
         return ser;
         break;
    case 'T':
         return thr;
         break;
    case 'W':
         return trp;
         break;
    case 'Y':
         return tyr;
         break;
    case 'V':
         return val;
         break;               
    default:
	 return oth;
	 break;
    }   
}
