/*
 *Greg Gysberg
 *November 11, 2005
 */

#include <stdio.h>
#include <math.h>

#define NUM_TEST      10   /* Maximum number of sequences in the test data.  */
#define MAX_SEQ_SIZE 500   /* Holds the longest sequence in either training or test set. */
#define NUM_AA        21   /* Number of possible amino acids.  */
#define MAX_ID        20   /* Maximum size of the name for the identification of the sequence. */

typedef enum
        {ala, arg, asn, asp, cys, gln, glu, gly, his, ile, leu, lys, met, phe, pro, ser, thr, trp, tyr, 
val, oth}
acid_t;

double train_aa_freq[NUM_AA];
char test_id[NUM_TEST][MAX_ID];
double test_aa_freq[NUM_TEST][NUM_AA];
double sum_of_squares[NUM_TEST];
int total_test[NUM_TEST];

void Read_Sequences(double train_aa_freq[NUM_AA], int *counter);

void Initialize_Arrays(void);

int AA(char aa);

void get_identifier(char test_id[NUM_TEST][MAX_ID]);

void Test_Sequences(double test_aa_freq[NUM_TEST][NUM_AA], int total_test[NUM_TEST]);

int
main ()
{
    double train_proportion[MAX_ID], test_proportion[NUM_TEST][MAX_ID], temp;

    int counter=0,
        i,
        j,
        index_of_min = 0;

    acid_t identifier = 0;

    for (i = 0;
         i <MAX_ID;
         i++) {
        train_proportion[i] = 0;
    }

    Initialize_Arrays();

    Read_Sequences(train_aa_freq, &counter);

    for (i = 0;
         i < MAX_ID;
         i++) {
        train_proportion[i] = train_aa_freq[identifier] / counter;
        identifier++;
    }

    printf("Amino Acid\tTrain Proportion\n");
    identifier = 0;
    for (i = 0;
         i < MAX_ID;
         i++) {
        printf("%5d\t\t%f\n", (acid_t)(identifier), train_proportion [i]);
        identifier++;
    }  
 
    get_identifier(test_id);

    Test_Sequences(test_aa_freq, total_test);
          
    for (i = 0;
         i < NUM_TEST;   
         i++) {
        for (j = 0;
             j < MAX_ID;
             j++) {
            test_proportion[i][j] = test_aa_freq[i][j] / total_test[i];
        }
    }
     
    printf("\n");
    printf("Test Sequence\tAmino Acid\tTest Proportion\n");
    for (i = 0;
         i < NUM_TEST;
         i++) {
        for (j = 0;
             j < MAX_ID;
             j++) {
            printf("%5c%2d%12c%2d%14c%5f\n", ' ', i, ' ', j, ' ', test_proportion[i][j]);
        }
        printf("\n");
    }
    
    printf("\n\n");
    printf("Sum of Squares\n");
    for (i = 0;
         i < NUM_TEST;
         i++) {
        for (j = 0;  
             j < MAX_ID;
             j++) {
            sum_of_squares[i] += pow((train_proportion[i] - test_proportion[i][j]), 2);
        }
        printf("%s\t%f\n", test_id[i], sum_of_squares[i]);
    }     
    
    printf("\n\nThe sum of squares ranked from closest to furthest from the training data proportions is:\n");
    for (i = 0;
         i < NUM_TEST - 1;
         i++) {
        for (j = i;
             j < NUM_TEST;
             j++) {
            if (sum_of_squares[j] < sum_of_squares[index_of_min]) {
                index_of_min = j;
            }
        }
        if (i != index_of_min) {
            temp = sum_of_squares[index_of_min];
            sum_of_squares[index_of_min] = sum_of_squares[i];
            sum_of_squares[i] = temp;
            }
        index_of_min = i + 1;
    }
         
    for (i = 0;
         i < NUM_TEST;
         i++) {
        printf("%f\n", sum_of_squares[i]);
    }
    
    return 0;
}
        
void
Read_Sequences(double train_aa_freq[NUM_AA], int *counter)
{
    char aa;
    FILE *inp;
    inp = fopen("train.dat", "r");
    
    while ((fscanf(inp, "%c", &aa)) != EOF) {
        if (aa == '>') {
            while (aa != '\n') {
                fscanf(inp, "%c", &aa);
            }
        }
        else {
            train_aa_freq[AA(aa)]+=1;
            if ((AA(aa)) != oth) 
                *counter+=1;
        }
    }
            
    fclose(inp);
}
             
void
Initialize_Arrays(void)
{        
    int i, j;  
    int a;
    int x, y;  
        
    for (i = 0;
         i < NUM_TEST;
         i++) {
        for (j = 0;
             j < NUM_AA;
             j++) {
            test_aa_freq [i][j] = 0;
        }
    }
    
    
    for (a = 0;
         a <NUM_AA;
         a++) {
        train_aa_freq[a] = 0;   
    }
             
         
    for (x = 0;
         x < NUM_TEST;
         x++) {
        for (y = 0;
             y < MAX_ID;
             y++) {
            test_id[x][y] = '\0';
        }
    }
             
    for (i = 0;
         i < NUM_TEST; 
         i++) {
        total_test[i] = 0;
    }
    
    for (i = 0;
         i < NUM_TEST;
         i++) {
        sum_of_squares[i] = 0;
    }
}
             
int
AA(char aa)
{
    switch (aa) {
    case 'A':
        return(ala);
        break;
         
    case 'R':
        return (arg);
        break;
         
    case 'N':  
        return (asn); 
        break; 
        
    case 'D':
        return (asp);
        break;
         
    case 'C':
        return (cys);
        break; 
         
    case 'Q':  
        return (gln);
        break;
    
    case 'E':  
        return (glu); 
        break; 
        
    case 'G':
        return (gly);
        break;
   
    case 'H':
        return (his);
        break;   
    
    case 'I':
        return (ile);
        break;
    
    case 'L':
        return (leu);
        break;
    
    case 'K':
        return (lys);
        break;
    
    case 'M':
        return (met);
        break;
    
    case 'F':
        return (phe);
        break;
    
    case 'P':
        return (pro);
        break;
    
    case 'S':
        return (ser);
        break;
    
    case 'T':
        return (thr);
        break;
    
    case 'W':
        return (trp);
        break;
    
    case 'Y':
        return (tyr);
        break;
    
    case 'V':
        return (val);
        break;
    
    default:
        return (oth);
    }
}
        
void
get_identifier(char test_id[NUM_TEST][MAX_ID])
{
    char aa = '\0';  
    FILE *inp;
    inp = fopen("test.dat", "r");
    int i = 0, j = 0;
        
    while ((fscanf(inp, "%c", &aa)) != EOF) {
        if (aa == '>') {
            fgets(test_id[i], MAX_ID, inp);
            i++;
        }
    }
}
        
void
Test_Sequences(double test_aa_freq[NUM_TEST][NUM_AA], int total_test[NUM_TEST])
{
    int i = -1;
    char aa;  
    FILE *inp;
    inp = fopen("test.dat", "r");
        
    while ((fscanf(inp, "%c", &aa)) != EOF) {
        if (aa == '>') {
            while (aa != '\n') {
                fscanf(inp, "%c", &aa);
            } 
            i++;
        }   
        else {
            test_aa_freq[i][AA(aa)]+=1;
            if ((AA(aa)) != oth) {
                total_test[i]+=1;
            }
        }
    }
}

